import "../../lib/jsonld.esm.js.js";
import jsonldRdfaParser from '../../lib/jsonld8-rdfa.esm.js';

jsonld.registerRDFParser('text/html', jsonldRdfaParser);

export const GLOBAL_CONTEXT = {
    "@vocab": "http://schema.org/",
    "@language": "fr"
}

/**
 * Fetch remote content data
 * @param {String|URL} documentUrl Document URL to fetch
 * @param {Object} context Optional context to format JSON
 * @returns JSON-LD of data extracted from the provided page
 */
export async function fetchDocumentData(documentUrl, context = GLOBAL_CONTEXT){
    let res = await fetch(documentUrl);

    if(!res.ok)
        throw new Error(`Network error ${res.status} ${res.statusText}`)

    if(!res.headers.get("Content-Type").startsWith("text/html"))
        throw new Error(`Invalid content type ${res.headers.get("Content-Type")}`)

    let content = await res.text()

    let remoteDoc = new DOMParser().parseFromString(content, "text/html");

    let base = document.createElement("base")
    base.href = documentUrl
    remoteDoc.head.append(base)

    return await getDocumentData(remoteDoc, context);
}

/**
 * Extract data from a page document
 * @param {Document} sourceDocument Page document to extract data from
 * @param {Object} context Optional context to format JSON
 * @returns JSON-LD of data extracted from the page
 */
export async function getDocumentData(sourceDocument = document, context = GLOBAL_CONTEXT) {
    let resource = sourceDocument.body.getAttribute("resource")

    if(resource)
        resource = new URL(resource, sourceDocument.baseURI)

    return await extractData(
        sourceDocument.documentElement,
        resource,
        context
    )
}

/**
 * Get JSON data from a RDFa Formatted element
 * @param {Element} sourceElement Source element to extract data from
 * @param {Object} context Optional context to format JSON
 * @returns JSON-LD of data extract from element
 */
export async function getData(sourceElement, context = GLOBAL_CONTEXT){
    let resource = sourceElement.getAttribute("resource");

    if(resource)
        resource = new URL(resource, sourceElement.baseURI)

    return await extractData(
        sourceElement,
        resource,
        context
    )
}

export function takeFirst(self){
    if(!self){
        return self
    }

    if(Array.isArray(self)){
        return self[0]
    } else {
        return self
    }
}

export function takeUrl(self){
    if(!self){
        return self
    }

    let first = takeFirst(self)
    return first["@id"]
}

async function extractData(baseElement, rootId, context){
    let sourceData = await jsonld.fromRDF(baseElement, {format: 'text/html'});
    
    if(rootId) {

        let frame = {
            "@id": rootId
        }

        if(context) {
            frame["@context"] = context
        }

        return await jsonld.frame(sourceData, frame)

    } else if (context) {

        return await jsonld.compact(sourceData, context)

    } else {

        return sourceData

    }
}